function [csmoothed,smoothWindow] = CEMOverlapWindow(funch,N1,N2,M1,M2)
% Smoothing embedding using overlapping windows
%
% Usage:
%   [csmoothed,smoothWindow] = CEMOverlapWindow(funch,N1,N2,M1,M2)
% Inputs:
%   funch     function handle for covariance
%   N1,N2     targeted domain size
%             NOTE: Code currently only supports square domain (N1=N2)
%   M1,M2     embedding domain size
%             NOTE: Current implementation only allows for odd number 
%             embedding sizes M1 and M2
%             NOTE: Code currently only supports square embedding domain
%             (M1=M2)
% Output:
%   csmoothed  embedded covariance, smoothed over the transition region
%              using overlapping windows
%   smoothWindow  window used in smoothing

% Check inputs
if mod(M1,2)==0 || mod(M2,2)==0,
    error('Code currently only supports odd number embedding sizes M1 and M2');
end

if N1~=N2,
    error('Code currently only supports square domain (N1=N2)');
end

if M1~=M2,
    error('Code currently only supports square embedding domain (M1=M2)');
end

if M1==2*N1-1,
    % no overlapping windows,
    % do standard embedding and return
    csmoothed = CEMfieldcov(funch,M1,M2);
    smoothedWindow = [];
    return;
end

% Generate window:
M1ext = 2*(M1-N1+1)-1;
M2ext = M1ext;
smoothWindow = GenWindow(N1,N2,M1ext,M2ext);
rho = (M1+1)/2;
epsilon = rho - N1;

% generate unsmoothed covariance on the domain of size M1ext-times-M2ext
c = CEMfieldcov(funch,M1ext,M2ext);

% do smoothing using overlapping windows
csmoothed1 = zeros(M1,M2);
csmoothed2 = zeros(M1,M2);
csmoothed3 = zeros(M1,M2);
csmoothed4 = zeros(M1,M2);

ind1x = 1:(rho+epsilon+1);
ind1y = 1:(rho+epsilon+1);
ind4x = (rho+epsilon+2):M1ext;
ind4y = (rho+epsilon+2):M2ext;
ind2x = ind4x;
ind2y = ind1y;
ind3x = ind1x;
ind3y = ind4y;

csmoothed1(ind1x,ind1y) = c(ind1x,ind1y).*smoothWindow(ind1x,ind1y);
csmoothed4(ind4x-2*epsilon,ind4y-2*epsilon) = c(ind4x,ind4y).*smoothWindow(ind4x,ind4y);
csmoothed2(ind2x-2*epsilon,ind2y) = c(ind2x,ind2y).*smoothWindow(ind2x,ind2y);
csmoothed3(ind3x,ind3y-2*epsilon) = c(ind3x,ind3y).*smoothWindow(ind3x,ind3y);

csmoothed = csmoothed1+csmoothed2+csmoothed3+csmoothed4;

% Copyright (c), Hannes Helgason, KTH, 2011