function [csmoothed,smoothWindow] = CEMWindow(funch,N1,N2,M1,M2,w)
% Smoothing embedding using non-overlapping windows
%
% Usage:
%   [csmoothed,smoothWindow] = CEMWindow(funch,N1,N2,M1,M2)
%   [csmoothed,smoothWindow] = CEMWindow(funch,N1,N2,M1,M2,w)
% Inputs
%   funch     function handle for covariance
%   N1,N2     targeted domain size
%             NOTE: Current implementation only allows for square-sized
%             domains (N1=N2)
%   M1,M2     embedding domain size
%             NOTE: Current implementation only allows for odd number 
%             embedding sizes M1 and M2
%             NOTE: Current implementation only allows for square-sized
%             domains (M1=M2)
%   w         optional input for providing window to use in smoothing,
%             if omitted, window will be generated using GenWindow.m
% Output
%   csmoothed  embedded covariance, smoothed over the transition region
%              using windows
%   smoothWindow  window used in smoothing

% Check inputs
if mod(M1,2)==0 || mod(M2,2)==0,
    error('Code currently only supports odd number embedding sizes M1 and M2');
end

if N1~=N2,
    error('Code currently only supports square domain (N1=N2)');
end

if M1~=M2,
    error('Code currently only supports square embedding domain (M1=M2)');
end

if M1==2*N1-1,
    % no overlapping windows,
    % do standard embedding and return
    csmoothed = CEMfieldcov(funch,M1,M2);
    smoothedWindow = [];
    return;
end

% generate window if needed
if nargin < 6,
    smoothWindow = GenWindow(N1,N2,M1,M2);
else
    % window was provided
    smoothWindow = w;
end
    
% generate unsmoothed covariance on the domain of size M1-by-M2
c = CEMfieldcov(funch,M1,M2);

% do smoothing
csmoothed = c.*smoothWindow;

% Copyright (c), Hannes Helgason, KTH, 2011