function [X, Xiidcopy] = FieldSynth(c,N1,N2)
% 2D field synthesis given embedded covariance
%
% Usage:
%   [X, Xiidcopy] = FieldSynth(c,N1,N2)
% Inputs:
%   c         embedded covariance
%             use CEMOverlapWindow.m or CEMfieldcov.m to generate it
%   N1,N2     targeted domain size
% Outputs:
%   X and Xiidcopy provide two independent realizations of the 2D field 
%   in a domain of size N1-by-N2
%
% NOTE: Code will exit with error if c has at least one negative eigenvalue.
%       One can check for negative eigenvalues by doing the following:
%         >> lambda = fft2(c);
%         >> lambdaReal = real(lambda);
%         >> numNeg = sum(sum(lambda < 0))
%       'numNeg' contains the number of negative eigenvalues.
%       For properly embedded covariance, all entries of 'lambda' are in
%       ideally real-valued -- taking the real-part simply done
%       to eliminate numerical errors.

% get embedding sizes
[M2,M1] = size(c); 

% Generate independent standard normals
Zreal = randn(M2,M1);
Zimag = randn(M2,M1);
Z = Zreal + i*Zimag;

% Calculate 2D DFT of c
lambda = fft2(c);

% impose lambda to be real-valued in case of numerical errors
lambda = real(lambda);

% check whether lambda has negative values
numNegativeVal = sum(sum(lambda < 0));
if numNegativeVal > 0,
    proportionNeg = numNegativeVal/numel(lambda);
    error(sprintf('Spectrum has negative values (proportion %0.2f)',proportionNeg));
end

% Generate field
sqrtLambdaZ = sqrt(lambda).*Z;

normFactor = 1/sqrt(M1*M2);
W = normFactor*fft2(sqrtLambdaZ);

X = real(W(1:N2,1:N1));
Xiidcopy = imag(W(1:N2,1:N1));

% Copyright (c), Hannes Helgason, KTH, 2011