function [Nmax,embeddingPossible] = FindMaxSize(funch,M,overlap)
% Find maximum possible size of synthesis domain for a fixed size
% of embedding domain
%
% Remark: Code currently only supports square-sized embedding domains,
%         
% 
% Usage:
%   [Nmax,embeddingPossible] = FindMaxSize(funch,M,overlap)
%   [Nmax,embeddingPossible] = FindMaxSize(funch,M)
% Inputs:
%   funch          function handle for covariance
%   M              sets the size of embedding domain, i.e. (M1,M2)=(M,M)
%                  NOTE: Current implementation only allows for odd number 
%                  embedding sizes M1 and M2.
%   overlap        flag to choose which kind of smoothing embedding to use.
%                  Possible values:
%                      'True' for overlapping windows
%                      'False' for nonoverlapping windows
%                  Recommended setting is 'True' as it should result in
%                  a larger synthesis domain size 'Nmax'
%                  Default value is 'True' if the parameter is not provided
%                  in the function call.
% Outputs:
%   Nmax           maximum possible size of synthesis domain for the given
%                  input parameters, i.e. (N1,N2)=(Nmax,Nmax)
%   embeddingPossible  flag telling whether embedding is possible for
%                  the provided input parameters; 'True' if embedding is
%                  possible, otherwise 'False'
%
% Example:
%    >> theta = 0.004;   
%    >> alpha = 1;
%    >> funch = @(n1,n2) poweredexp(n1,n2,theta,alpha);
%    >> M = 2001;
%    >> [Nmax,embeddingPossible] = FindMaxSize(funch,M);

if nargin < 3,
    overlap = true;
end


if mod(M,2)==0,
    error('Code currently only supports odd number embedding size M');
end
M1 = M;
M2 = M1;

% first try standard embedding
N = (M+1)/2;
N1 = N;
N2 = N1;
    
if overlap,
    csmoothed = CEMOverlapWindow(funch,N1,N2,M1,M2);
else
    csmoothed = CEMWindow(funch,N1,N2,M1,M2);
end

% Calculate spectrum and check for negative eigenvalues
lambda = real(fft2(csmoothed));
minLambda = min(min(lambda));

if minLambda >= 0,
    % standard embedding works so return
    Nmax = N;
    embeddingPossible = true;
    return;
else
	Nupperbound = N;
    embeddingPossible = false;
end
Nlowerbound = 2;
Nmax = 1;

% Loop until we have found the optimal size    
while Nlowerbound <= Nupperbound,
    
    Nmaxtmp = round((Nupperbound+Nlowerbound)/2);
    N1 = Nmaxtmp; N2 = N1;
    
    if overlap,
        csmoothed = CEMOverlapWindow(funch,N1,N2,M1,M2);
    else
        csmoothed = CEMWindow(funch,N1,N2,M1,M2);
    end
    % Calculate spectrum and check for negative eigenvalues
    lambda = real(fft2(csmoothed));
    minLambda = min(min(lambda));
    
    if minLambda >= 0,
        % have new lower bound and embedding possible
        Nmax = Nmaxtmp;
        Nlowerbound = Nmaxtmp+1;
        embeddingPossible = true;
    else
        Nupperbound = Nmaxtmp-1;
    end
end
if Nmax==2 & embeddingPossible == false,
    Nmax = 1;
end


% Copyright (c), Hannes Helgason, KTH, 2011