function [w,epsilon,rho] = GenWindow(N1,N2,M1,M2)
% Generate smoothing window for embedding 
%
% Usage:
%   w = GenWindow(N1,N2,M1,M2)
% Inputs
%   N1,N2     targeted domain size
%             NOTE: Code currently only supports square domain (N1=N2)
%   M1,M2     embedding domain size
%             NOTE: Code currently only supports odd number embedding sizes
%             M1 and M2 and square embedding domain (M1=M2)
% Output
%   w         window to use in smoothing
%   epsilon, rho  parameters used in creating the window
%
% The window is created by convolving an indicator function over square 
% with a mollifier -- see functions boxindicator.m and mollifier2D.m. 
% This routine will automatically select the parameters for boxindicator.m
% and mollifier2D.m based on the values of N1, N2, M1 and M2.

% Check inputs
if mod(M1,2)==0 || mod(M2,2)==0,
    error('Code currently only supports odd number embedding sizes M1 and M2');
end

if N1~=N2,
    error('Code currently only supports square domain (N1=N2)');
end

if M1~=M2,
    error('Code currently only supports square embedding domain (M1=M2)');
end

% Set parameters for window function
epsilon = floor((M1+1)/2 - N1)/2;
rho = N1+epsilon;

funchBox = @(t1,t2) boxindicator(t1,t2,rho);
funchMoll = @(t1,t2) mollifier2D(t1,t2,epsilon);

% generate window
boxwindow = CEMfieldcov(funchBox,M1,M2);
moll = CEMfieldcov(funchMoll,M1,M2);
tmp = fft2(boxwindow).*fft2(moll);
w = ifft2(tmp);
w = abs(real(w));
w = w/w(1,1); % normalize

% Copyright (c), Hannes Helgason, KTH, 2011