function val = poweredexp(n1,n2,theta,alpha)
% Powered exponential covariance function
%   r(n) = exp( -(theta * ||n||)^alpha )
% where ||n|| is the l2 norm of n=(n_1,n_2) and
% theta>0, and 0 < alpha <=2
%
% Usage:
%   val = poweredexp(n1,n2,theta,alpha)
% Inputs:
%   n1        matrix of indices for 1st coordinate
%   n2        matrix of indices for 2nd coordinate
%             NOTE: n1 and n2 have to be of equal dimensions
%             where entry n2(k,j) corresponds to entry n1(k,j)
%   theta     positive scalar
%   alpha     scalar satisfying 0 < alpha <= 2
% Output:
%   val       matrix with values of covariance at the indices given
%             in n1 and n2
%             val is of the same dimensions as n1 and n2 where
%             entry val(k,j) corresponds to the pair (n1(k,j),n2(k,j))

% Checking arguments:
if (alpha <= 0) || (alpha > 2),
    error('Input argument "alpha" has to satisfy 0 < alpha <= 2.')
end
if theta <= 0,
    error('Input argument "theta" has to satisfy theta > 0.')
end

tmp = n1.^2 + n2.^2;
tmp = theta*sqrt(tmp);
val = exp(-tmp.^alpha);


% Copyright (c), Hannes Helgason, KTH, 2011