function val = poweredexpW(n1,n2,theta,alpha,W)
% Anisotropic covariance in 2D with exponential-type decay
%   r(n) = exp(-alpha ||n||_W^beta)
% where W symmetric and positive definite,
% n=(n1,n2), ||n||_W^2 = n W n^T,
% alpha>0, and 0 < beta<=2
%
% Usage:
%   val = poweredexpW(n1,n2,theta,alpha,W)
% Inputs
%   n1        matrix of indices for 1st coordinate
%   n2        matrix of indices for 2nd coordinate
%             NOTE: n1 and n2 have to be of equal dimensions
%             where entry n2(k,j) corresponds to entry n1(k,j)
%   theta     positive scalar
%   alpha     scalar satisfying 0 < beta <= 2
%   W         symmetric and positive definite 2-by-2 matrix
% Output
%   val       matrix with values of covariance at the indices given
%             in n1 and n2
%             val is of the same dimensions as n1 and n2 where
%             entry val(k,j) corresponds to the pair (n1(k,j),n2(k,j))

% Checking arguments:
if (alpha <= 0) || (alpha > 2),
    error('Input argument "alpha" has to satisfy 0 < alpha <= 2.')
end
if theta <= 0,
    error('Input argument "theta" has to satisfy theta > 0.')
end

tmp = W(1,1)*n1.^2 + W(2,2)*n2.^2 + 2*W(1,2)*n1.*n2;
weigthedNorm = sqrt(tmp);
val = exp(-(theta*weigthedNorm).^alpha);

% Copyright (c), Hannes Helgason, KTH, 2011